
local pl = require "pl.pretty"
require "version"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
client.InitSocket(nil, adapter)
local api = {
	device_type = "",
	driving_mode = "",
	network = {}
}

local function GetProxy(objPath, service, loc, timeout, socket_addr)
	local proxy
	local cli

	cli = client.New( service, loc, socket_addr, timeout or 5 ) --5 seconds timeout.
	if not cli then
		print( "CO", "Fail to create connection to service " .. (loc or "local") .. "/" .. (service or "nil") )
		return nil
	end
	proxy = cli:CreateProxy(objPath, true) --Exclusive
	if not proxy then
		print( "CO", "Fail to create object proxy for '" .. (objPath or "unknown") .. "'" )
		return nil
	end
	return proxy
end

local function GetDrivingMode()
    local proxy = GetProxy( "/", "codec" )
	if proxy then
		local r, mode = proxy.GetMode()
        proxy:Destroy()
		if r and mode then
			if mode == "decoding" or mode == "decoder" then
				api.driving_mode = "decoder"
			else
				api.driving_mode = "encoder"
			end
            return api.driving_mode
		end
	end
    return ""
end

local function SerialNumber()
	local serial = "000000000"
    local f = io.open( "/boot/SERIAL_NO", "r" )
    if f then
        serial = f:read("*l")
        f:close()
	else
		local t = io.open( "/etc/SERIAL_NO", "r" )
		if t then
			serial = t:read("*l")
			t:close()
		end
    end
    return serial
	--for test
	-- return "20220333"
end

local function GetDeviceName()
	local filename = "/etc/hostname"
    local f = io.open( filename, "r" )
	if f then
		local r = f:read("*l")
		f:close()
        if r then
		    return r
        end
    end
    return ""
end

function api.Init( device_type )
	api.device_type = device_type
end

function api.SerialNumber()
    return SerialNumber()
end

function api.DeviceName()
    return GetDeviceName()
end

function api.Reboot()
	local proxy = GetProxy( "/", "systemctrl" )
	if proxy then
		proxy.Reboot(2)
		proxy:Destroy()
	end
end

function api.Reset()
	local proxy = GetProxy( "/", "systemctrl" )
	if proxy then
		proxy.Reset()
		proxy:Destroy()
	end
end

function api.GetBasicInformation(info)
    info.device_name  = GetDeviceName()
    info.device_mode  = "IO"
	info.device_type  = api.device_type
    info.driving_mode = GetDrivingMode()
	info.serial_number    = SerialNumber()
	info.software_version = _G.SOFTWARE_VERSION
	info.hardware_version = _G.HARDWARE_VERSION
	info.web_index = info.driving_mode
    return true, info
end

function api.GetEncoderInfo(info)
	--判断运行模式
	if api.driving_mode == "" or api.driving_mode == "decoder" then 
		return false, info 
	end

	local proxy = GetProxy( "/", "codec" )
	if proxy then
		-- audio_channels: 2
		-- audio_format: "48000Hz/Stereo"
		-- audio_sampling: 48000
		-- audio_signal: "embedded"
		-- bitrate: 130329
		-- frame_rate: 59.94
		-- interlaced: false
		-- resolution: "1920x1080@59.94 Hz"
		-- video_signal: true
		-- xRes: 1920
		-- yRes: 1080
		local r, status = proxy.GetStatus()
		if r and status then
			info.audio = {}
			info.video = {}
			local audio_signal
			if not status.audio_signal then
				audio_signal = "none"
			elseif status.audio_signal == "hdmi" or status.audio_signal == "sdi" or status.audio_signal == "digital" or status.audio_signal == "embedded" then
				if api.device_type == "N3" or api.device_type == "N30" then
					audio_signal = "SDI"
				else
					audio_signal = "HDMI"
				end
			elseif status.audio_signal == "line" or status.audio_signal == "linein" or status.audio_signal == "line-in" or status.audio_signal == "mic" or status.audio_signal == "micin" or status.audio_signal == "mic-in" or status.audio_signal == "analog" then
				audio_signal = "analog"
			else
				audio_signal = status.audio_signal
			end
			local audio_chn = tonumber(status.channels) or 2
			local audio_channel_names = {
				[0] = "Auto",
				[1] = "Mono",
				[2] = "Stereo"
			}
			
			info.audio.audio_format   = (status.sampling or 48000) .. "Hz/" .. (audio_channel_names[audio_chn] or ("" .. audio_chn .. " CH"))
			info.audio.audio_channels = audio_chn
			info.audio.audio_sampling = status.sampling
			info.audio.audio_signal   = audio_signal
			if status.video_signal then
				if api.device_type == "N3" or api.device_type == "N30" then
					info.video.video_signal = "SDI"
				else
					info.video.video_signal = "HDMI"
				end
				info.video.video_encode = "NDI-FULL"
			else
				info.video.video_signal = "NULL"
			end
			info.video.interlaced     = status.interlaced
			info.video.resolution = status.resolution
			info.video.frame_rate = status.frame_rate
			info.video.bitrate    = status.bitrate
			info.video.xRes = status.xRes
			info.video.yRes = status.yRes
			return true, info
		end
	end
	return false, info 
end

function api.GetCaptureInfo(info)
	return false, info
end

function api.GetDecoderInfo(info)
	if api.driving_mode == "" or api.driving_mode == "encoder" then 
		return false, info 
	end
	local proxy = GetProxy( "/", "codec" )
	if proxy then
		local r, status = proxy.GetStatus()
		proxy:Destroy()
		-- audio: "48.0KHz / 2CH"
		-- audio_channels: 2
		-- audio_sampling: 48000
		-- bitrate: 132375
		-- codec: "SHQ2"
		-- deInterlace: false
		-- frame_rate: 60.03
		-- interlaced: false
		-- ip: "192.168.35.57"
		-- name: "DESKTOP-0ERC3PJ (Intel UHD Graphics 630 2)"
		-- online: true
		-- ptz: 1
		-- resolution: "1920x1080p 60Hz"
		-- url: "192.168.35.57:6041"
		-- video_mode: "hdmi"
		-- warning: ""
		-- web: ""
		-- xRes: 1920
		-- yRes: 1080
		if r and status then
			info.audio = {}
			info.video = {}
			info.audio.audio 			= status.audio
			info.audio.audio_channels = status.audio_channels
			info.audio.audio_sampling = status.audio_sampling
			if api.device_type == "N3" or api.device_type == "N30" then
				info.audio.audio_mode = "SDI"
			else
				info.audio.audio_mode = "HDMI"
			end
			info.bitrate = status.bitrate
			
			info.video.deInterlace = status.deInterlace
			info.video.frame_rate  = status.frame_rate
			info.video.resolution = status.resolution
			info.video.video_mode = status.video_mode
			info.video.xRes = status.xRes
			info.video.yRes = status.yRes

			if status.codec:match("^H") then
				info.codec   = "NDI-HX"
			else
				info.codec   = "NDI-FULL"
			end
			info.interlaced  = status.interlaced
			info.ip 	= status.ip
			info.name 	= status.name
			info.online = status.online
			info.ptz 	= status.ptz
			info.url 		= status.url
			info.warning 	= status.warning
			info.web  = status.web
			
			return true, info
		end
	end
	return false, info
end

function api.MediaStream( info )
	return false, info
end

function api.GetSnapshoot()
	return false, ""
end

function api.UpgradeDevice(file)
    local proxy = GetProxy("/", "systemctrl", "local", 30)
    if not proxy then
        print("Can't Get the systemctrl proxy !!!")
        return
    end

    local is_ok, skip = proxy.ProcessFirmware(file)
    proxy:Destroy()
    return is_ok and skip
end


return api